//
// Copyright 2013, Authenticade LLC
// MIT License - See LICENSE file for details.
//

var CK_HOME = "https://www.cryokey.com";

// These placeholders will contain the actual ticket handlers.
var CK_HEAD = document.getElementsByTagName('head')[0];
var CK_HANDLER = document.createElement('script');
var CK_SUBMIT = null;
var CK_ATTEMPTS = 0;

// Insert the initially empty script block.
CK_HEAD.appendChild(CK_HANDLER);

// Get an HTTP connection object.
function ck_get_connector()
{
  try { return new XMLHttpRequest(); } catch (error) { }
  try { return new ActiveXObject("Msxml2.XMLHTTP"); } catch (error) { }
  try { return new ActiveXObject("Microsoft.XMLHTTP"); } catch (error) { }
  throw new Error("Could not create HTTP request object.");
}

// Attempt to clear cached authentication information.
function ck_logout(_url)
{
  if (/MSIE ([0-9]{1,}[\.0-9]{0,})/.test(navigator.userAgent))
    document.execCommand('ClearAuthenticationCache');
  if (window.crypto && window.crypto.logout)
    window.crypto.logout();
  window.location = _url;
}

// Attempt a multi-factor authentication. We do the ticketing and
// then submit the standard form with the ticket ID.
function ck_multi(_url, _ip)
{
  CK_SUBMIT = function(_tid)
  {
    var form = document.forms['loginform'];
    var action = form.getAttribute("action");
    if (/\?/.test(action))
      form.setAttribute("action", action + "&cktid=" + escape(_tid));
    else
      form.setAttribute("action", action + "?cktid=" + escape(_tid));
    form.submit();
  }

  if (/(iPad|iPhone|iPod)/.test(navigator.userAgent))
  {
    CK_ATTEMPTS = 1;
    ck_ticket(_url, _ip);
    return;
  }
  if (/MSIE ([0-9]{1,}[\.0-9]{0,})/.test(navigator.userAgent))
  {
    document.execCommand('ClearAuthenticationCache');
    CK_ATTEMPTS = 3;
    ck_ticket(_url, _ip);
    return;
  }
  if (window.crypto && window.crypto.logout)
  {
    window.crypto.logout();
    CK_ATTEMPTS = 3;
    ck_ticket(_url, _ip);
    return;
  }

  var flush = ck_get_connector();
  flush.onreadystatechange = function()
  {
     if (flush.readyState == 4)
     {
       CK_ATTEMPTS = 3;
       ck_ticket(_url, _ip);
     }
  }
  flush.open("GET", CK_HOME + "/public/flush", true);
  flush.send();
}

// Initiate a log in attempt. Clear existing credentials before the
// attempt.
function ck_initiate(_url, _ip, _pass)
{
  CK_SUBMIT = function(_tid)
  {
    if (/\?/.test(_pass))
      window.location = _pass + "&cktid=" + escape(_tid);
    else
      window.location = _pass + "?cktid=" + escape(_tid);
  }

  if (/(iPad|iPhone|iPod)/.test(navigator.userAgent))
  {
    CK_ATTEMPTS = 1;
    ck_ticket(_url, _ip);
    return;
  }
  if (/Android/.test(navigator.userAgent))
  {
    // Note: the CryoKey Android application must be available.
    window.location = CK_HOME + "/private/ticket.php?mode=plain&url=" + escape(_url) + "&ip=" + escape(_ip) + "&pass=" + escape(_pass);
    return;
  }
  if (/MSIE ([0-9]{1,}[\.0-9]{0,})/.test(navigator.userAgent))
  {
    document.execCommand('ClearAuthenticationCache');
    CK_ATTEMPTS = 3;
    ck_ticket(_url, _ip);
    return;
  }
  if (window.crypto && window.crypto.logout)
  {
    window.crypto.logout();
    CK_ATTEMPTS = 3;
    ck_ticket(_url, _ip);
    return;
  }

  var flush = ck_get_connector();
  flush.onreadystatechange = function()
  {
     if (flush.readyState == 4)
     {
       CK_ATTEMPTS = 3;
       ck_ticket(_url, _ip);
     }
  }

  flush.open("GET", CK_HOME + "/public/flush", true);
  flush.send();
}

// Acquire a ticket from CryoKey's ticketing module.
function ck_ticket(_url, _ip)
{
  var now = new Date().getTime() / 1000;

  CK_ATTEMPTS--;
  CK_HEAD.removeChild(CK_HANDLER);

  CK_HANDLER = document.createElement('script');
  CK_HANDLER.type = "text/javascript";
  CK_HANDLER.onabort = function()
  {
    ck_logout(_url);
  }
  CK_HANDLER.onerror = function()
  {
    if (CK_ATTEMPTS > 0)
      ck_ticket(_url, _ip);
    else
      ck_logout(_url);
  }

  CK_HANDLER.src = CK_HOME + "/private/ticket.php?mode=js&url=" + escape(_url) + "&ip=" + escape(_ip) + "&skew=" + now;

  CK_HEAD.appendChild(CK_HANDLER);
}

// Complete authentication by passing the TicketID to the target
// service.
function ck_login(_tid)
{
  if (CK_SUBMIT)
    CK_SUBMIT(_tid);
}
